# Function to build a function to put inside compute
build_compute_workflow <- function(recipe) {
  ## TODO: Sort out system and order
  ## Alternatively, this function could just be defined by the user
  # Define function
  ## TODO: Define returns
  my_compute_function = function(recipe, hcst, obs, fcst = NULL) {
    modules <- tolower(strsplit(recipe$Run$startR_workflow$modules,
                                ", | |,")[[1]])
    data <- list(hcst = hcst, obs = obs, fcst = fcst)
    skill_metrics <- NULL
    for (module in modules) {
      if (module == "calibration") {
        data <- calibrate_datasets(recipe, data)
      } else if (module == "anomalies") {
        data <- compute_anomalies(recipe, data)
      } else if (module == "skill") {
        skill_metrics <- compute_skill_metrics(recipe, data)
        # return_list <- c(return_list, skill_metrics = skill_metrics)
      }
    }
    ## TODO: Define what to return depending on functions.
    ## Also, maybe return an additional variable to determine if the output
    ## needs to be converted back to s2dv_cube or what.
    return(list(data$hcst, data$obs, skill_metrics))
  }
  info(recipe$Run$logger, "##### COMPUTE FUNCTION DEFINED #####")
  return(my_compute_function)
}

convert_to_s2dv_cube <- function(new_cube, original_cube) {
  ## TODO: Make function more general? 
  ## TODO: Make sure all the metadata is created correctly
  attr(new_cube, "Variables") <- attr(original_cube, "Variables")
  attr(new_cube, "FileSelectors") <- attr(original_cube, "FileSelectors")

  class(new_cube) <- "startR_array"
  new_cube <- as.s2dv_cube(new_cube)
  return(new_cube)
}

run_compute_workflow <- function(recipe, data) {

  modules <- tolower(strsplit(recipe$Run$startR_workflow$modules,
                              ", | |,")[[1]])
  # Step 1: Define the function
  ## TODO: Sort out order
  my_compute_function <- build_compute_workflow(recipe)
  # Step 2: Define the call to compute()
  # Create target dimensions
  ## TODO: Refine this part
  target_dims <- list(hcst = c('sday', 'sweek', 'syear', 'ensemble'),
                      obs = c('sday', 'sweek', 'syear'))
  if (!is.null(data$fcst)) {
    target_dims <- c(target_dims,
                     list(fcst = c('sday', 'sweek', 'syear', 'ensemble')))
  }

  # Create output dimensions
  ## TODO: Add several conditions depending on user requests
  output_dims <- list(hcst = c('sday', 'sweek', 'syear', 'ensemble'),
                      obs = c('sday', 'sweek', 'syear'))
  if (!is.null(data$fcst)) {
    output_dims <- c(output_dims,
                     list(fcst = c('sday', 'sweek', 'syear', 'ensemble')))
  }
  if ("skill" %in% modules) {
    output_dims <- c(output_dims,
                     list(skill = c('metric', 'syear')))
  }
  ## TODO: Create output dimensions dynamically
  ## TODO: Add use attributes
  ## TODO: Handle skill output
  step <- Step(fun = my_compute_function,
               target_dims = target_dims,
               output_dims = output_dims)
  wf <- AddStep(inputs = list(hcst = data$hcst, obs = data$obs),
                              step,
                              recipe = recipe)
  res <- Compute(wf$hcst,
                 chunks = recipe$Run$startR_workflow$chunk_along)

  info(recipe$Run$logger,
       "##### COMPUTE SECTION ENDED, CONVERTING TO s2dv_cube #####")
  # Step 3: Convert result to s2dv_cube and return it
  ## TODO: fcst
  hcst <- convert_to_s2dv_cube(res$hcst, data$hcst)
  obs <- convert_to_s2dv_cube(res$obs, data$obs)
  ## TODO: Transform skill metrics back into a list
  metric_list <- c("rpss", "rpss_significance") ## to be removed
  skill <- .drop_dims(res$skill)
  # skill_metrics <- apply(seq(dim(skill)[1]), function(x) {res$skill[x, , , ]})
  
  info(recipe$Run$logger,
       "##### DATA RETURNED AS A NAMED LIST #####")
  return(list(hcst = hcst, obs = obs, skill_metrics = skill_metrics))
}

